import 'package:flutter_mylibrary/http/dio_utils.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

import '../request_info.dart';
import '../view_state_model.dart';

abstract class ViewStateRequestListModel<T> extends ViewStateModel{

  /// 页面数据
  List<T> list = [];

  RequestInfo getRequestInfo();


  onCompleted(List<T> data) {}

  void initData({bool retry = false}) async{
    RequestInfo requestInfo = getRequestInfo();
    if (retry) {
      setBusy();
    }
    list = await DioUtils.getInstance().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onFinish: (){
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
        },
        onError: isInit? null : (e){
          setError(e, e.message, message: e.message);
        }
    ).then((res){
      try{
        return res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data);
      }catch(e){
        setError(HttpError(HttpError.UNKNOWN, e.toString()), e.toString());
        return null;
      }
    });
    if (!isError) {
      setIdle();
    }
    if (isIdle && isEmpty(list)) {
      ///请求无异常但是数据为空需要设置为empty
      setEmpty();
    }
    onCompleted(list);
  }

  ///获取数据
  Future<List<T>> getData(){
    RequestInfo requestInfo = getRequestInfo();
    return DioUtils.getInstance().request(
        requestInfo?.method,
        requestInfo?.url,
        data: requestInfo?.data,
        publicMethod: false,
        onFinish: (){
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
        },
        onError: isInit? null : (e){
          setError(e, e.message, message: e.message);
        }
    ).then((res) => res.data is List? List<T>.from(res.data.map((e) => requestInfo?.handleRes(e))).toList() : requestInfo?.handleRes(res.data));
  }
}